<?php
/******************************************************************************
* e107 and coppermine bridge file                                             *
*******************************************************************************
*                                                                             *
*   Written by mygoggie                                                       *
*   $Id: e107.inc.php                                                         *
*   $Source: www.purplescroll.com/ v$                                         *
*   $Revision: 1.1.0 $                                                        *
*   $Date: 2011-02-10 $                                                       *
*   $Author: mygoggie $                                                       *
*                                                                             *
* @copyright Copyright  2007-2010, purplescroll.com, All rights reserved.    *
*                                                                             *
* This file is part of the purple_cpg_bridge plugin for e107                  *
*                                                                             *
* The purple_cpg_bridge plugin for e107 is free software: you can             *
* redistribute it and/or modify it under the terms of the GNU General         *
* Public License as published by the Free Software Foundation, either         *
* version 3 of the License, or (at your option) any later version.            *
*                                                                             *
* The purple_cpg_bridge plugin for e107 is distributed in the hope that it    *
* will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty *
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with the purple_cpg_bridge plugin for e107.                           *
*                                                                             *
* If not, see <http://www.gnu.org/licenses/>.                                 *
*                                                                             *
******************************************************************************/

/******************************************************************************
  Coppermine version: 1.5.12
******************************************************************************/

if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

if (isset($bridge_lookup))
{
    $default_bridge_data[$bridge_lookup] = array(
        'full_name' => 'e107 v0.7.24+ bridge for Coppermine 1.5.x',
        'short_name' => 'e107_coppermine',
        'support_url' => 'http://www.purplescroll.com/',
        'full_forum_url_default' => 'http://www.your_e107_site.com/',
        'full_forum_url_used' => 'mandatory,not_empty,no_trailing_slash',
        'relative_path_to_config_file_default' => '../',
        'relative_path_to_config_file_used' => 'lookfor,e107_config.php',
        'use_post_based_groups_default' => '1',
        'use_post_based_groups_used' => 'radio,1,0',
    );
    
} else {

    // Switch that allows overriding the bridge manager with hard-coded values
    define('USE_BRIDGEMGR', 1);

    // Include files we require
    require_once 'bridge/udb_base.inc.php';

    class cpg_udb extends core_udb {

        //Declare some variables we need in this class
        static private $arrayBridge = array();  // The array with bridge variables we will use

        // ------------------------------------------------------------------------------>
        // PHP5+ Constructor
        // ------------------------------------------------------------------------------>
        function __construct() 
        {
            global $BRIDGE;
            
            if (!USE_BRIDGEMGR) { // the vars that are used when bridgemgr is disabled

                // URL of your e107 site
                $this->boardurl = 'http://localhost';

                // local path to your e107 config file
                require_once('../e107_config.php');

                $this->use_post_based_groups = 1;

            } else { // the vars from the bridgemgr
                $this->boardurl = $BRIDGE['full_forum_url'];                              // The path to site root

                require_once($BRIDGE['relative_path_to_config_file'].'e107_config.php');  // Get the file e107_config.php

                $this->use_post_based_groups = $BRIDGE['use_post_based_groups'];
            }

            // Set some flags we require
            $this->multigroups              = TRUE; // User can belong to more than one group (e107 userclass)
            $this->group_overrride          = TRUE; //
            $this->use_post_based_groups    = TRUE; //

            
            // Database connection settings
            $this->db = array(
                'name'      => $mySQLdefaultdb,
                'host'      => $mySQLserver,
                'user'      => $mySQLuser,
                'password'  => $mySQLpassword,
                'prefix'    => $mySQLprefix
            );
            
            // Board table names
            $this->table = array(
                'users'     => 'user',
                'groups'    => 'userclass_classes',
                'sessions'  => 'session',
                'config'    => 'core',
                'plugin'    => 'purple_cpg_bridge'
            );

            // Derived full table names
            $this->usertable = '`' . $this->db['name'] . '`.' . $this->db['prefix'] . $this->table['users'];
            $this->groupstable =  '`' . $this->db['name'] . '`.' . $this->db['prefix'] . $this->table['groups'];
            $this->sessionstable =  '`' . $this->db['name'] . '`.' . $this->db['prefix'] . $this->table['sessions'];
            $this->configtable = '`' . $this->db['name'] . '`.' . $this->db['prefix'] . $this->table['config'];
            $this->plugintable = '`' . $this->db['name'] . '`.' . $this->db['prefix'] . $this->table['plugin'];
            
            // Table field names
            $this->field = array(
                'username'              => 'user_name',         // name of 'username' field in users table
                'user_id'               => 'user_id',           // name of 'id' field in users table
                'is_admin'              => 'user_admin',        // whether user is admin
                'admin_perms'           => 'user_perms',        // admin permissions
                'password'              => 'user_password',     // name of 'password' field in users table
                'email'                 => 'user_email',        // name of 'email' field in users table
                'regdate'               => 'user_join',         // name of 'registered' field in users table
                'lastvisit'             => 'user_lastvisit',    // last time user logged in
                'active'                => 'user_ban',          // is user account active?
                'location'              => "''",                // name of 'location' field in users table
                'website'               => "''",                // name of 'website' field in users table
                'usertbl_group_id'      => 'user_class',        // name of 'group id' field in users table
                'grouptbl_group_id'     => 'userclass_id',      // name of 'group id' field in groups table
                'grouptbl_group_name'   => 'userclass_name',    // name of 'group name' field in groups table
            );
            
            // Pages to redirect to
            $this->page = array(
                'edituserprofile'   => '/usersettings.php',
                'login'             => '',
                'logout'            => '?logout',
                'register'          => '/signup.php',
                'users'             => '/user.php',
                // 'viewuserprofile'       => '/user.php?id.',  // NOT USED NOW
            );

            // Load our userclass array
            $this->getUserclasses();

            // Get and set the Admin userclass ID
            $this->getAdminUserclass();
            
//print "<br /> Admin group = ";
//var_dump($this->admingroups);

            // Set the guest group userclass to the same as that of e107
            $this->guestgroup = 252;

//print "<br /> Category Map = ";
//var_dump($this->arrayBridge);

            // Map various Coppermine items to corresponding e107
            if(empty($this->arrayBridge['flagMapped']))
            {
                // Set the Coppermine usergroup mapping for 'User galleries' to the e107 Coppermine userclass
                $this->setCategoryMap();

                // Set menu items to point to e107
                $this->setCustomMenuItems();
            }



            // Connect to db
            $this->connect();

        } // End of constructor


        // ------------------------------------------------------------------------------>
        // NAME: getUserclasses
        // ------------------------------------------------------------------------------>
        // Function:
        // Get the e107 userclass that is applicable to Coppermine
        //
        // Parameters:
        // none
        //
        // Return:
        // nothing
        // ------------------------------------------------------------------------------>
        function getUserclasses()
        {
            // Connect to db
            $this->connect();

            // Get the specified e107 userclass 
            $query = "SELECT * FROM {$this->plugintable}";
            $result = cpg_db_query($query, $this->link_id);

            while ($row = mysql_fetch_assoc($result))
            {
                $this->arrayBridge[$row['preference']] = $row['value'];
            }

            mysql_free_result($result);

        } // EO method getUserclasses()


        // ------------------------------------------------------------------------------>
        // NAME: getAdminUserclass
        // ------------------------------------------------------------------------------>
        // Function:
        // Get the e107 admin userclass that will be used for Coppermine admins
        //
        // Parameters:
        // none
        //
        // Return:
        // nothing
        // ------------------------------------------------------------------------------>
        function getAdminUserclass()
        {
            // Connect to db
            $this->connect();

            // Declare admin array
            $this->admingroups = array();
            
            // Set the admin userclass
            $adminUserclass = $this->arrayBridge['adminUserclass'];

            // Get and set Coppermine admin group
            $query = "SELECT * FROM {$this->groupstable} WHERE `userclass_name` = '{$adminUserclass}'";
            $result = cpg_db_query($query, $this->link_id);
            $row = mysql_fetch_assoc($result);

//print "<br /> Admin group1 = ";
//var_dump($row);

            // If we have users in the group, set the admin group
            if ($row['userclass_id'])
            {
                $this->admingroups = array($row['userclass_id']);
            }

            mysql_free_result($result);

        } // EO method getAdminUserclass()


        function collect_groups()
        {
            // Declare variables we need
            $udb_groups = array();

            // New array - Make the userclass ID the key and the userclass description the value
            $udb_groups[$this->arrayBridge['userUserclassID']] = $this->arrayBridge['userUserclass'];
            $udb_groups[$this->arrayBridge['adminUserclassID']] = $this->arrayBridge['adminUserclass'];

            // Now get the rest of the userclasses from e107
            $query ="SELECT * FROM {$this->groupstable} WHERE `userclass_name` NOT IN ('PRIVATEFORUM1','PRIVATEMENU', '".$udb_groups[$this->arrayBridge['userUserclassID']]."', '".$udb_groups[$this->arrayBridge['adminUserclassID']]."')";
            $result = cpg_db_query($query, $this->link_id);

            $udb_groups[$this->guestgroup]='Guests';

            while ($row = mysql_fetch_assoc($result))
            {
                // don't include CPM_ADMIN_GROUP Group
                if (!in_array($row[$this->field['grouptbl_group_id']], $this->admingroups))
                {
                    //$udb_groups[$row[$this->field['grouptbl_group_id']]+100] = $row[$this->field['grouptbl_group_name']];
                    $udb_groups[$row[$this->field['grouptbl_group_id']]] = $row[$this->field['grouptbl_group_name']];
                }
            }

//print "<br /> Groups = ";
//var_dump($udb_groups);

            return $udb_groups;

        } // End of method collect_groups


        // ------------------------------------------------------------------------------>
        // NAME: setUserGalleries
        // ------------------------------------------------------------------------------>
        // Function:
        // Set the default Coppermine class ID for the category 'User galleries' to the
        // e107 userclass for coppermine users.
        // This is set in the Coppermine database, table 'categorymap'
        //
        // Parameters:
        // none
        //
        // Return:
        // nothing
        // ------------------------------------------------------------------------------>
        function setCategoryMap( )
        {
            //Set global variables
            global $CONFIG;

            // The default CPG $cid for 'User galleries' is 1, so set it
            $cid = 1;

            // The e107 Coppermine userclass ID
            $value = $this->arrayBridge['userUserclassID'];

            // Connect to db
            $this->connect();

            // Add the mapping
            // First check if mapping exists
            $query = "SELECT `cid` FROM {$CONFIG['TABLE_CATMAP']} WHERE `cid` = '1'";
            $result = cpg_db_query($query);
            $row = mysql_fetch_assoc($result);
//print "<br /> Category Map function = ";
//var_dump($row);
            // If the mapping does not exist add it otherwise update it
            if (empty($row))
            {
                cpg_db_query("INSERT INTO {$CONFIG['TABLE_CATMAP']} (cid, group_id) VALUES ({$cid}, {$value})");

            } else {

                // Update the settings
                cpg_db_query("UPDATE {$CONFIG['TABLE_CATMAP']} SET `group_id`='{$value}' WHERE `cid` = '{$cid}'");

            }

            // Add the mapped flag to DB and array
            cpg_db_query("UPDATE {$this->plugintable} SET `value`='1' WHERE `preference` = 'flagMapped'");
            $this->arrayBridge['flagCategoryMapped'] = TRUE;

//print "<br /> Category Map function = ";
//var_dump($this->arrayBridge);

        } // End of method setUserGalleries()


        // ------------------------------------------------------------------------------>
        // NAME: authenticate
        // ------------------------------------------------------------------------------>
        // Function:
        // Authenticates the user
        // This method overrides the function in the file udb_base.inc.php
        //
        // Parameters:
        // none
        //
        // Return:
        // nothing
        // ------------------------------------------------------------------------------>
        function authenticate()
        {
            global $USER_DATA;

            if (!($auth = $this->session_extraction()) && !($auth = $this->cookie_extraction()) && !($auth = $this->post_extraction())) {
                $this->load_guest_data();
            } else {
                list ($id, $cookie_pass) = $auth;
                $f = $this->field;
            
//print "<br /> ID : ".$id;
//print "<br /> ID : ".$cookie_pass;
//print "<br /> In Authenticate : ";
//var_dump($f);
//print "<br /> usergroupstable : ".$this->usergroupstable;

                if (isset($this->usergroupstable))
                {
                    $sql = "SELECT u.{$f['user_id']} AS id, u.{$f['username']} AS username, {$f['password']} AS password, ug.{$f['usertbl_group_id']} AS group_id "
                            . "FROM {$this->usertable} AS u, {$this->usergroupstable} AS ug "
                            . "WHERE u.{$f['user_id']}=ug.{$f['user_id']} AND u.{$f['user_id']}='$id'";
                } else {
                    $sql = "SELECT u.{$f['user_id']} AS id, u.{$f['username']} AS username, {$f['password']} AS password, u.{$f['usertbl_group_id']} AS group_id "
                            . "FROM {$this->usertable} AS u LEFT JOIN {$this->groupstable} AS g ON u.{$f['usertbl_group_id']}=g.{$f['grouptbl_group_id']} "
                            . "WHERE u.{$f['user_id']}='$id'";
                }
//print "<br /> SQL : ".$sql;
                $result = cpg_db_query($sql, $this->link_id);
//print "<br /> Result : ".$result;
                if (mysql_num_rows($result))
                {
                    $row = mysql_fetch_assoc($result);
                    mysql_free_result($result);

//print "<br /> Password : ".$row['password'];
                    $db_pass = $this->udb_hash_db($row['password']);
                
//print "<br /> Password : ".$db_pass;
                    if ($db_pass === $cookie_pass) {
                        $this->load_user_data($row);
                    } else {
                        $this->load_guest_data();
                    }
                } else {
                    $this->load_guest_data();
                }
            }
//print "<br /> User data : ";
//var_dump($USER_DATA);

            $user_group_set = '(' . implode(',', $USER_DATA['groups']) . ')';

            $USER_DATA = array_merge($USER_DATA, $this->get_user_data($USER_DATA['groups'][0], $USER_DATA['groups'], $this->guestgroup));

            if ($this->use_post_based_groups){
//print "<br /> 2 User data : ";
//var_dump($USER_DATA);

//print "<br /> Admin groups : ";
//var_dump($this->admingroups);
                $USER_DATA['has_admin_access'] = 0;

                // Check if user's group is in the list of CPG admin groups
                // If so set flag to TRUE, else set to FALSE
                foreach($USER_DATA['groups'] as $group_id)
                {
//print "<br /> Check if admin access - Value: ";
//print $group_id;
                
                    if (in_array($group_id,$this->admingroups))
                    {
                        $USER_DATA['has_admin_access'] = 1;
                    }
                }

            //$USER_DATA['has_admin_access'] = (in_array($USER_DATA['groups'] - 100,$this->admingroups)) ? 1 : 0;
            
//print "<br /> Check if admin access : ";
//print in_array($USER_DATA['groups'][0] - 100,$this->admingroups);
//print "<br /> Has admin access : ";
//print $USER_DATA['has_admin_access'];

            } else {
                $USER_DATA['has_admin_access'] = ($USER_DATA['groups'][0] == 1) ? 1 : 0;
            }

            $USER_DATA['can_see_all_albums'] = $USER_DATA['has_admin_access'];

            // avoids a template error
            if (!$USER_DATA['user_id']) { 
                $USER_DATA['can_create_albums'] = 0;
            }

            // For error checking
            $CONFIG['TABLE_USERS'] = '**ERROR**';

            define('USER_ID', $USER_DATA['user_id']);
            define('USER_NAME', addslashes($USER_DATA['user_name']));
            define('USER_GROUP', $USER_DATA['group_name']);
            define('USER_GROUP_SET', $user_group_set);
            define('USER_IS_ADMIN', $USER_DATA['has_admin_access']);
            define('USER_CAN_SEND_ECARDS', (int)$USER_DATA['can_send_ecards']);
            define('USER_CAN_RATE_PICTURES', (int)$USER_DATA['can_rate_pictures']);
            define('USER_CAN_POST_COMMENTS', (int)$USER_DATA['can_post_comments']);
            define('USER_CAN_UPLOAD_PICTURES', (int)$USER_DATA['can_upload_pictures']);
            define('USER_CAN_CREATE_ALBUMS', ((int)$USER_DATA['can_create_albums'] || (int)$USER_DATA['can_create_public_albums']));
            define('USER_CAN_CREATE_PRIVATE_ALBUMS', (int)$USER_DATA['can_create_albums']);
            define('USER_CAN_CREATE_PUBLIC_ALBUMS', (int)$USER_DATA['can_create_public_albums']);
            define('USER_ACCESS_LEVEL', (int)$USER_DATA['access_level']);

            $this->session_update();

        } // EO method authenticate()


        // ------------------------------------------------------------------------------>
        // NAME: session_extraction
        // ------------------------------------------------------------------------------>
        // Function:
        // Extract the user id, name, group from a session cookie
        // This method will be completed at a later stage.  e107 currently does not write session detail 
        // to a DB table, so this needs to be written as part of the plugin.This is set in the Coppermine database, table 'categorymap'
        //
        // Parameters:
        // none
        //
        // Return:
        // $array - An array with all session details
        // ------------------------------------------------------------------------------>
        function session_extraction()
        {
            return FALSE;
        } // EO method session_extraction()


        // ------------------------------------------------------------------------------>
        // NAME: cookie_extraction
        // ------------------------------------------------------------------------------>
        // Function:
        // Extract the user id, name, group from a cookie
        // This method will be completed at a later stage.  e107 currently does not write session detail 
        // to a DB table, so this needs to be written as part of the plugin.This is set in the Coppermine database, table 'categorymap'
        //
        // Parameters:
        // none
        //
        // Return:
        // $array - An array with all cookie details
        // ------------------------------------------------------------------------------>
        function cookie_extraction()
        {
            $superCage = Inspekt::makeSuperCage();

            // Get cookie name
            $query = "SELECT `e107_value` FROM {$this->configtable} WHERE `e107_name` = 'SitePrefs'";
            $result = cpg_db_query($query, $this->link_id);
            $row = mysql_fetch_assoc($result);

            mysql_free_result($result);

            // Get tracking type
            preg_match("/user_tracking\' => \'(.*?)\'/", $row['e107_value'], $tracking_type);
            
            // Check if user tracking is cookies
            if ($tracking_type[1] === 'cookie')
            {
                // Load the cookie name into the array
                preg_match("/cookie_name\' => \'(.*?)\'/", $row['e107_value'], $cookie);

                // Get the cookie and split it into an array with USERID and PASSWORD as elements
                if ($superCage->cookie->keyExists($cookie[1]))
                {
                    $array = array_map('addslashes', explode(".", $superCage->cookie->getRaw($cookie[1]), 2));
                    return $array;

                } else {

                    // No cookie found
                    return FALSE;
                }

            } else {

                // Tracking not by cookies
                return FALSE;
            }
        } // EO method cookie_extraction()


        // ------------------------------------------------------------------------------>
        // NAME: get_groups
        // ------------------------------------------------------------------------------>
        // Function:
        // Get groups of which user is member
        //
        // Parameters:
        // $row - The database row read
        //
        // Return:
        // $userclass - An array with all the userclasses
        // ------------------------------------------------------------------------------>
        function get_groups($row)
        {
            if (!$row['group_id'])
            {
                return $this->arrayBridge['userUserclassID'];
            }

            $userclass = explode(",", $row['group_id']);
/*
            foreach ($userclass as $class_id => $value)
            {
                $userclass[$class_id]+=100;
            }
*/
//print "<br /> Get groups function = ";
//var_dump($userclass);

            return $userclass;

        } // EO method get_groups()


        // ------------------------------------------------------------------------------>
        // NAME: udb_hash_db
        // ------------------------------------------------------------------------------>
        // Function:
        // Definition of actions required to convert a password from user database form to cookie form
        //
        // Parameters:
        // $password - The password to convert
        //
        // Return:
        // $password - The converted password
        // ------------------------------------------------------------------------------>
        function udb_hash_db($password)
        {
            // DB password must be double hashed to get it to e107 cookie form, so hash and return it
            $password = md5($password);

            return $password;

        } // EO method udb_hash_db()


        // ------------------------------------------------------------------------------>
        // NAME: setCustomMenuItems
        // ------------------------------------------------------------------------------>
        // Function:
        // Set custom menu items to point to e107 pages
        //
        // Parameters:
        // none
        //
        // Return:
        // none
        // ------------------------------------------------------------------------------>
        function setCustomMenuItems()
        {
            global $CONFIG;
            
            // Set the home page link to e107 home page
            $homepath = $this->arrayBridge['e107_path'];
            $forumMenuName = $this->arrayBridge['e107_forum_name'];
            $pathForum = $this->arrayBridge['e107_forum_path'];
            $table = $CONFIG['TABLE_CONFIG'];

            // Set 'HOME' menu item path
            cpg_db_query("UPDATE $table SET value='{$homepath}' WHERE name='home_target'");

            // Create 'Forum' menu item
            cpg_db_query("UPDATE $table SET value='{$forumMenuName}' WHERE name='custom_lnk_name'");

            // Set 'Forum' menu item path
            cpg_db_query("UPDATE $table SET value='{$pathForum}' WHERE name='custom_lnk_url'");
        }

        // ------------------------------------------------------------------------------>
        // Various page redirection functions
        // ------------------------------------------------------------------------------>

        // Home page
        function home_page()
        {
            $this->redirect($this->page['home']);
        } // EO method


        // Register page
        function register_page()
        {
            $this->redirect($this->page['register']);
        } // EO method


        // Login page
        function login_page()
        {
            $this->redirect($this->page['login']);
        } // EO method


        // Logout page
        function logout_page()
        {
            $this->redirect($this->page['logout']);
        } // EO method


        // View list of users
        function view_users()
        {
            $this->redirect($this->page['users']);
        } // EO method


        // Edit user profile
        function view_profile()
        {
            $this->redirect($this->page['edituserprofile']);
        } // EO method


    } // End of class

    // All done, so let's start the bridge
    $cpg_udb = new cpg_udb;
    
} // End of if block
?>