<?php
/******************************************************************************
* purple_cpg_bridge class handler file                                        *
*******************************************************************************
*                                                                             *
*   Written by mygoggie                                                       *
*   $Id: purple_cpg_bridge_class.php                                          *
*   $Source: www.purplescroll.com/ v$                                         *
*   $Revision: 1.1.0 $                                                        *
*   $Date: 2011-02-10 $                                                       *
*   $Author: mygoggie $                                                       *
*                                                                             *
* @copyright Copyright  2007-2010, purplescroll.com, All rights reserved.    *
*                                                                             *
* This file is part of the purple_cpg_bridge plugin for e107                  *
*                                                                             *
* The purple_cpg_bridge plugin for e107 is free software: you can             *
* redistribute it and/or modify it under the terms of the GNU General         *
* Public License as published by the Free Software Foundation, either         *
* version 3 of the License, or (at your option) any later version.            *
*                                                                             *
* The purple_cpg_bridge plugin for e107 is distributed in the hope that it    *
* will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty *
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with the purple_cpg_bridge plugin for e107.                           *
*                                                                             *
* If not, see <http://www.gnu.org/licenses/>.                                 *
*                                                                             *
******************************************************************************/

// Protect the file from direct access
if(!defined("e107_INIT")){ exit; }

class purple_cpg_bridge
{
    static private $purple_cpg_bridge_instanceConfig = null;

    var $array_Purple_cpg_bridge = array();

    // ------------------------------------------------------------------------------
    // PHP5+ Constructor
    // ------------------------------------------------------------------------------
    function __construct()
    {

        // Remember - Load the plugin's constants in e_module.php

        // Declare path to our admin language file
        $plugin_ad_lan = PURPLE_CPG_BRIDGE_LANGUAGES_PATH.e_LANGUAGE."/admin/lan_admin_purple_cpg_bridge_".e_LANGUAGE.".php";

        // File_exists() cache answers
        clearstatcache();

        // Load the language files if not loaded
        if (!defined('LAN_AD_PURPLE_CPG_BRIDGE_FLAG'))
        {
            if (e_LANGUAGE != "English" && file_exists($plugin_ad_lan))
            {
                @include_lan($plugin_ad_lan);

            } else {

                // Assume that the English language file is always present
                @include_lan(PURPLE_CPG_BRIDGE_LANGUAGES_PATH."English/admin/lan_admin_purple_cpg_bridge_English.php");
            }
        }

    // End of constructor
    }


    // ------------------------------------------------------------------------------
    // get_instance method
    // ------------------------------------------------------------------------------
    static public function get_instance()
    {
        // If there is not already an instance of this class, instantiate one.
        if (self::$purple_cpg_bridge_instanceConfig == null){
            self::$purple_cpg_bridge_instanceConfig = new purple_cpg_bridge();
        }

        return self::$purple_cpg_bridge_instanceConfig;

    } // EO method



    // ------------------------------------------------------------------------------>
    // NAME: loadPrefs
    // ------------------------------------------------------------------------------>
    // @Function:
    // Load purple_login pref values from e107 database
    //
    // @Parameters:
    // None
    //
    // @Return:
    // TRUE on success, FALSE on fail
    // ------------------------------------------------------------------------------>
    function loadPrefs()
    {
        // Declare variables
        global  $sql;

        // Retrieve preferences from core database
        if (!is_object($sql))
        {
            $sql = new db;
        }

        $num_rows = $sql->db_Select('purple_cpg_bridge');

        // If no value defined in core DB then insert default preference values
        if (empty($num_rows))
        {
            // Load default values
            $this->loadDefaults();

            foreach($this->array_Purple_cpg_bridge as $preference => $value)
            {
                $sql->db_Insert('purple_cpg_bridge', '"'.$preference.'", "'.$value.'"');
            }

            $sql->db_Select('purple_cpg_bridge');

        } else {

            // Load values into array
            while($row = $sql->db_Fetch())
            {
                $this->array_Purple_cpg_bridge[$row['preference']] = $row['value'];
            }
        }
    } // EO method loadPrefs()



    // ------------------------------------------------------------------------------>
    // NAME: loadDefaults
    // ------------------------------------------------------------------------------>
    // @Function:
    // Load default pref values as per the files in config/ directory
    //
    // @Parameters:
    // Path as input
    //
    // @Return:
    // TRUE on success, FALSE on fail
    // ------------------------------------------------------------------------------>
    function loadDefaults()
    {
        // Load configuration file
        require_once(PURPLE_CPG_BRIDGE_CONFIG_PATH.'purple_cpg_bridge_configuration.php');
 
        // Get our array as configured in purple_login_configuration.php
        $array_object = new purple_cpg_bridge_config();

    } // EO method loadDefaults()



    // ------------------------------------------------------------------------------>
    // NAME: savePrefs
    // ------------------------------------------------------------------------------>
    // @Function:
    // Save changes in purple_cpg_bridge values to purple_login database
    //
    // @Parameters:
    // None
    //
    // @Return:
    // TRUE on success, FALSE on fail
    // ------------------------------------------------------------------------------>
    // ------------------------------------------------------------------------------>
    // Save changes in purple_login_pref values to purple_login database
    // ------------------------------------------------------------------------------>
    function savePrefs()
    {
        // Declare stuff we need
        global $sql, $tp;

        // save preferences to database
        if (!is_object($sql))
        {
            $sql = new db;
        }

        // Save changed values
        foreach($this->array_Purple_cpg_bridge as $preference => $value)
        {
            // Write data into database storage format
            $message = $sql->db_Update('purple_cpg_bridge', "value='".$tp->toDB($value, true)."' WHERE preference='{$preference}'");
        }

        return $message;

  } // EO method savePrefs()


    // ------------------------------------------------------------------------------>
    // NAME: copyBridgeFile
    // ------------------------------------------------------------------------------>
    // @Function:
    // Copy the e107 bridge file to the bridge directory of the Coppermine installation
    //
    // @Parameters:
    // Path as input
    //
    // @Return:
    // TRUE on success, FALSE on fail
    // ------------------------------------------------------------------------------>
    function copyBridgeFile($path)
    {

//print "<br /> Path to copy = ";var_dump($path);

        // Move and copy files
        if (!copy(PURPLE_CPG_BRIDGE_FILES_PATH.'e107.inc.php', $path.'bridge/e107.inc.php'))
        {
            // Pop warning window up
            $text = LAN_AD_PURPLE_CPG_BRIDGE_HANDLER_CLASS_A01.PURPLE_CPG_BRIDGE_FILES_PATH.'e107.inc.php'.LAN_AD_PURPLE_CPG_BRIDGE_HANDLER_CLASS_A06.$path.LAN_AD_PURPLE_CPG_BRIDGE_HANDLER_CLASS_A07;
            message_handler('P_ALERT', $text, $line = 0, $file = "");
        }

/* Now override the function authenticate() in e107.inc.php

        // Rename the existing file
        $successMV = rename($path.'bridge/udb_base.inc.php' , $path.'bridge/udb_base.inc.php.purple');

        // Copy the new file
        $successCP = copy(PURPLE_CPG_BRIDGE_FILES_PATH.'udb_base.inc.php', $path.'bridge/udb_base.inc.php');

        // Move and copy files
        if (!$successMV || !$successCP)
        {
            // Pop warning window up
            $text = LAN_AD_PURPLE_CPG_BRIDGE_HANDLER_CLASS_A01.PURPLE_CPG_BRIDGE_FILES_PATH.'udb_base.inc.php'.LAN_AD_PURPLE_CPG_BRIDGE_HANDLER_CLASS_A06.$path.LAN_AD_PURPLE_CPG_BRIDGE_HANDLER_CLASS_A07;
            message_handler('P_ALERT', $text, $line = 0, $file = "");
        }
*/

    } // EO method copyBridgeFile()

}// EO class


// End of PHP file
?>