<?php
/******************************************************************************
* purple_files class handler file                                             *
*******************************************************************************
*                                                                             *
*   Written by mygoggie                                                       *
*   $Id: eg_files_class.php                                                   *
*   $Source: www.purplescroll.com/ v$                                         *
*   $Revision: 1.1.0 $                                                        *
*   $Date: 2010/10/20 $                                                       *
*   $Author: mygoggie $                                                       *
*                                                                             *
* @copyright Copyright  2007-2010, purplescroll.com, All rights reserved.    *
*                                                                             *
* This file is part of the purple_cpg_bridge plugin for e107                  *
*                                                                             *
* The purple_files handler for e107 is free software: you can                 *
* redistribute it and/or modify it under the terms of the GNU General         *
* Public License as published by the Free Software Foundation, either         *
* version 3 of the License, or (at your option) any later version.            *
*                                                                             *
* The purple_files handler for e107 is distributed in the hope that it        *
* will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty *
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with the purple_fancybox plugin for e107.                             *
*                                                                             *
* If not, see <http://www.gnu.org/licenses/>.                                 *
*                                                                             *
******************************************************************************/

// Protect the file from direct access
if(!defined("e107_INIT")){ exit; }

// --------------------------------------------------------------------------------
// Description:  - This class handles all the functions that relate to file and
//                 directory operations.
//               - It handles the quirks of IIS as well
// -------------------------------------------------------------------------------- 

// Load some files we require
@require_once(e_HANDLER."file_class.php");

class purple_file extends e_file
{
    //Declare some variables we need in this class
    static private $purple_file_instanceConfig = NULL;  // The purple_file instance
    static private $purple_classObject = NULL;          // The purple_class object
    
    // ------------------------------------------------------------------------------
    // PHP5+ Constructor
    // ------------------------------------------------------------------------------  
    public function __construct() 
    {
        // Check if new purple_class object must be declared
        // Flag is set in constants file
        if (PURPLE_HANDLER_CLASS_FLAG)  
        {
            // Include the handler class file
            require_once(PURPLE_CPG_BRIDGE_HANDLER_PATH.'purple_handlers_class.php');

            // Declare new object
            $this->purple_classObject = purple_handlers::get_instance();     // The object containing all the basic purple methods
        }

        // Check if the operating system is Winslows
        if(strtoupper(substr(PHP_OS, 0, 3)) == 'WIN')
        { 
            define('WINDOWS_OS', TRUE);
        } else {
            define('WINDOWS_OS', FALSE);
        }


        // This section makes sure the $_SERVER['DOCUMENT_ROOT'] variable is set
        // Usually required because of dumb IIS admins not doing there job
        if(!isset($_SERVER['DOCUMENT_ROOT']))
        {
            if(isset($_SERVER['SCRIPT_FILENAME']))
            {
                $_SERVER['DOCUMENT_ROOT'] = str_replace( '\\', '/', substr($_SERVER['SCRIPT_FILENAME'], 0, 0-strlen($_SERVER['PHP_SELF'])));
            }
        }

        if(!isset($_SERVER['DOCUMENT_ROOT']))
        {
            if(isset($_SERVER['PATH_TRANSLATED']))
            {
                $_SERVER['DOCUMENT_ROOT'] = str_replace( '\\', '/', substr(str_replace('\\\\', '\\', $_SERVER['PATH_TRANSLATED']), 0, 0-strlen($_SERVER['PHP_SELF'])));
            }
        }
        // $_SERVER['DOCUMENT_ROOT'] is now set - you can use it as usual...

        /*
        * REQUEST_URI for IIS Servers
        * Version: 1.0
        * Guaranteed to provide Apache-compliant $_SERVER['REQUEST_URI'] variables
        * Please see full documentation at 
        *
        * Copyright NeoSmart Technologies 2006-2007
        * Code is released under LGPL and maybe used for all private and public code
        * Questions/Support requests go at http://neosmart.net/forums/?getforum=5
        *
        * Product URI: http://neosmart.net/dl.php?id=7
        */

        //This file should be located in the same directory as php.exe or php5isapi.dll

        if (isset($_SERVER['HTTP_REQUEST_URI']))
        {
            $_SERVER['REQUEST_URI'] = $_SERVER['HTTP_REQUEST_URI'];
            //Good to go!

        } else {

            //Someone didn't follow the instructions!
            if(isset($_SERVER['SCRIPT_NAME']))
            {
                $_SERVER['HTTP_REQUEST_URI'] = $_SERVER['SCRIPT_NAME'];
            } else {
                $_SERVER['HTTP_REQUEST_URI'] = $_SERVER['PHP_SELF'];
            }

            if($_SERVER['QUERY_STRING'])
            {
                $_SERVER['HTTP_REQUEST_URI'] .=  '?' . $_SERVER['QUERY_STRING'];
            }

            //WARNING: This is a workaround!
            //For guaranteed compatibility, HTTP_REQUEST_URI *MUST* be defined!
            //See product documentation for instructions!
            $_SERVER['REQUEST_URI'] = $_SERVER['HTTP_REQUEST_URI'];

        }

    } // EO constructor

    // ------------------------------------------------------------------------------
    // get_instance method
    // ------------------------------------------------------------------------------
    static public function get_instance($location = NULL)
    {
        // If there is not already an instance of this class, instantiate one.
        if (self::$purple_file_instanceConfig == null){
            self::$purple_file_instanceConfig = new purple_file($location);
        }

        return self::$purple_file_instanceConfig;

    } // EO method



    // ------------------------------------------------------------------------------
    // Method: canonicalize()
    // Description: Canonicalize a path
    // Parameters: $address - Address to process
    // Returns: $address - Processed path
    // Author: mygoggie - www.purplescroll.com
    // ------------------------------------------------------------------------------
    function canonicalize($address)
    {
        $address = str_replace("//", "/", $address);
        $address = explode('/', $address);
        $keys = array_keys($address, '..');
        foreach($keys as $keypos => $key) array_splice($address, $key - ($keypos * 2 + 1), 2);
        $address = implode('/', $address);

        return preg_replace(',([^.])\./,', '\1', $address);
    } // EO of method - canonicalize()


    // ------------------------------------------------------------------------------
    // Method: getRealPath()
    // Description: Get the real path to base server installation. Required for file and directory handling
    // Parameters: $value - Path to process
    // Returns: $output - Processed path
    // Author: mygoggie - www.purplescroll.com
    // ------------------------------------------------------------------------------
    function getRealPath($value)
    {
        $output = '';

        if(($path = realpath($value)) && $path != $value)
        {
            $output =  $this->purple_classObject->forwardslashToBackslash($this->purple_classObject->addTrailingSlash($path));

        } else {

            $output =$this->purpleRealPath($value);

        }

        return $output;
    } // EO of method - getRealPath()


    // ------------------------------------------------------------------------------
    // Method: purpleRealPath()
    // Description: Sorts the mess out that dumb IIS server admins create
    // Parameters: $value - Path to process
    // Returns: $finalpath - Processed path
    // Author: mygoggie - www.purplescroll.com
    // ------------------------------------------------------------------------------
    function purpleRealPath($path)
    {

        if(strpos($path, ':/') !== false)
        {
            return $path;
        }

        // Get path to Site root
        $path = $this->purple_classObject->backslashToForwardslash($path);
        $path = $this->pathToSiteRoot($path);

        // Check if path passed was root i.e. '/'
        if (strpos($path,"/") === 0 && strlen($path == 1) )
        {
            return $path;

        } else {

            if (WINDOWS_OS)
            {

            if($_SERVER['APPL_PHYSICAL_PATH'] == '' || !isset($_SERVER['APPL_PHYSICAL_PATH']))
            {
                if($_SERVER['SCRIPT_FILENAME'] == '' || !isset($_SERVER['SCRIPT_FILENAME']))
                {
                    // Pop warning window up
                    $text = LAN_HANDLER_HANDLERS_A03;
                    message_handler('P_ALERT', $text, $line = 0, $file = "");

                } else {

                    $string1 = $_SERVER['SCRIPT_FILENAME'];
                }

                if($_SERVER['PHP_SELF'] == '' || !isset($_SERVER['PHP_SELF']))
                {
                    if($_SERVER['SCRIPT_NAME'] == '' || !isset($_SERVER['SCRIPT_NAME']))
                    {
                        // Pop warning window up
                        $text = LAN_HANDLER_HANDLERS_A02;
                        message_handler('P_ALERT', $text, $line = 0, $file = "");

                    } else {

                        $string2 = $_SERVER['SCRIPT_NAME'];
                    }

                } else {

                    $string2 = $_SERVER['PHP_SELF'];
                }

                $base = $this->diff_array(FALSE, NULL, NULL, $string1, $string2);

            } else {

                // WINDOWS
                $base = $_SERVER['APPL_PHYSICAL_PATH'];
            }

            // Set path to base
            $path = $base.e_HTTP.$path;

            // Canonicalize the path
            $finalpath = $this->purple_classObject->forwardslashToBackslash($this->canonicalize($this->purple_classObject->backslashToForwardslash($this->purple_classObject->fixWinslowsPath($path))));

            } else {

            // UNIX 
            $base = str_replace($_SERVER['SCRIPT_NAME'],'', $_SERVER['SCRIPT_FILENAME']);

            // Set path to base
            $path = $base.e_HTTP.$path;

            // Canonicalize the path
            $finalpath = $this->canonicalize($path);

            }
        }

        return ($finalpath);

    } // EO of method - purpleRealPath()


  // ------------------------------------------------------------------------------
  // Compile relative path between two specified directories
  // Concatenate will build from dir_one to root to dir_two
  // ------------------------------------------------------------------------------
  function buildPath($dir_one, $dir_two, $concatenate=TRUE)
  {
    global $tp;

    // Strip whitespaces out - keep normal spaces
    $dir_one = $this->purple_classObject->trimlrm($dir_one, "\t\n\r\0\x0B");
    $dir_two = $this->purple_classObject->trimlrm($dir_two, "\t\n\r\0\x0B");

    // Replace e_* constants  
    $path_one = $tp->replaceConstants($dir_one, full);
    $path_two = $tp->replaceConstants($dir_two, full);

    // Convert slashes to forward slashes
    $path_one = $this->purple_classObject->backslashToForwardslash($path_one);
    $path_two = $this->purple_classObject->backslashToForwardslash($path_two);

    // Determine the path to the site root from the directories specified
    $root_path_one = $this->purple_classObject->removePrependedSlash($this->pathToSiteRoot($path_one));
    $root_path_two = $this->purple_classObject->removePrependedSlash($this->pathToSiteRoot($path_two));

    // Build relative paths to site root
    $relative_path_one = $root_path_one == '/' ? './' : str_repeat("../",  substr_count($root_path_one, "/"));
    $relative_path_two = $root_path_two == '/' ? './' : str_repeat("../",  substr_count($root_path_two, "/"));

    // Create array from string
    $arrayDirs_path_one = explode('/', $root_path_one);
    $arrayDirs_path_two = explode('/', $root_path_two);

    // Remove blank values from array
    $arrayCleanedDirs_path_one = $this->purple_classObject->removeValueFromArray($arrayDirs_path_one, '');
    $arrayCleanedDirs_path_two = $this->purple_classObject->removeValueFromArray($arrayDirs_path_two, '');

    // Build the relative path between the two directories
    if ($concatenate)
    {

      // Calculate difference between the two relative paths
      $arrayDiff_path = array_intersect($arrayCleanedDirs_path_two, $arrayCleanedDirs_path_one);

      // Remove identical dirs from both paths
      foreach($arrayDiff_path as $segment)
      {
        array_shift($arrayCleanedDirs_path_one);
        array_shift($arrayCleanedDirs_path_two);

      }

      // Count number of dirs to same starting point of both dirs and add this to path two
      foreach($arrayCleanedDirs_path_one as $segment)
      {
        array_unshift($arrayCleanedDirs_path_two, '..');
      }

      // Implode and make final path
      $constructed_path = implode('/', $arrayCleanedDirs_path_two);
      $final_path = $this->purple_classObject->addTrailingSlash($constructed_path); 

    } else {

      $final_path = $relative_path_one;
    }

    // Check if directory exists
    $path = $this->purpleRealPath($final_path);

    if (!is_dir($path))
    {

      // Build new path between the two directories
      $final_path = $this->buildPath($dir_one, purple_wysiwyg_DEFAULT_USER_PATH, TRUE);

      // Pop warning window up
      $text = LAN_HANDLER_FILEMANAGER_A06.$path_two.LAN_HANDLER_FILEMANAGER_A07;
      $text .= "<br /><br />";
      $text .= LAN_HANDLER_FILEMANAGER_A08.$final_path.LAN_HANDLER_FILEMANAGER_A09;
      message_handler('P_ALERT', $text, $line = 0, $file = "");
    }

    return $final_path;

  } // EO method buildPath()


    // ------------------------------------------------------------------------------
    // Method: pathToSiteRoot()
    // Description: Renders a cleaned path to site root
    // Parameters: $path - Path to process
    //             $returnArray - Flag to return the path as an array
    // Returns: $finalPath - Processed path
    // Author: mygoggie - www.purplescroll.com
    // ------------------------------------------------------------------------------
    function pathToSiteRoot($path, $returnArray = FALSE)
    {
        // Create array from string
        $arrayDirs = explode('/', $path);

        // Remove blank values from array
        $arrayCleanedDirs = $this->purple_classObject->removeValueFromArray($arrayDirs, '');
        $arrayCleanedDirs = $this->purple_classObject->removeValueFromArray($arrayCleanedDirs, '.');
        $arrayCleanedDirs = $this->purple_classObject->removeValueFromArray($arrayCleanedDirs, '..');

        if($returnArray)
        {
            return $arrayCleanedDirs;

        } else {

            // Return the slashes.
            $finalpath = implode($arrayCleanedDirs,'/');
            $finalpath = $this->purple_classObject->addTrailingSlash($finalpath);

            return $finalpath;
        }
    } // EO of method pathToSiteRoot()


 
    // ------------------------------------------------------------------------------
    // Method: createDir()
    // Description: Create recursive directories
    // Parameters: $path - Directory to create - must be absolute path
    //             $recursively - Create dirs recursively
    //             $mask - The CHMOD value
    //             $dirOwner - IN the Unix format 'The owner of the dir:The group that has access to the dir'
    // Returns: $result - Returns TRUE on success or FALSE on failure.
    // Author: mygoggie - www.purplescroll.com
    // ------------------------------------------------------------------------------
    public function createDir($path, $recursively=FALSE, $mask=NULL, $dirOwner=NULL)
    {
        $result = TRUE;

        // Strip whitespaces out - keep normal spaces
        $path = $this->purple_classObject->trimlrm($path, "\t\n\r\0\x0B");
        //print "<br />1 createDir: Path = ".$path;  
        if($recursively)
        {
            $path = str_replace("/", DIRECTORY_SEPARATOR, $path);

            $pathNodes = explode(DIRECTORY_SEPARATOR, $path);
            $p = '';

            foreach($pathNodes as $node)
            {
                clearstatcache();
                if (!empty($node))
                {
                    if(strpos($node, ':') && WINDOWS_OS)
                    {
                        // Is a Windows path and is most probably the first node
                        $p .= $node;

                    } else {

                        $p .= DIRECTORY_SEPARATOR . $node;

                    }
                }

                if(!empty($node) && !file_exists($p) )
                {
                    $result = $this->createDir($p, FALSE, $mask, $dirOwner);
                    if(!($result))
                    {
                        return $result;
                    }
                }
            }

        } else {

            clearstatcache();
            if(!file_exists($path))
            {
                if (WINDOWS_OS)
                {
                    // WINSLOWS wants two forward slashes to create the folder
                    $path = str_replace(DIRECTORY_SEPARATOR,DIRECTORY_SEPARATOR.DIRECTORY_SEPARATOR, $path);
                }

                $result = mkdir($path);

                if ($mask)
                {
                    chmod($path, intval($mask,8));
                }

                if($dirOwner)
                {
                    $dirOwners = explode(":", $dirOwner);
                    if(!empty($dirOwners[0]))
                        chown($path, $dirOwners[0]);
                    if(!empty($dirOwners[1]))
                        chgrp($path, $dirOwners[1]);
                }

            } else {

                $result = TRUE;

            }
        }

        return $result;

    } // EO of method createDir()


    // ------------------------------------------------------------------------------
    // Method: getDirList()
    // Description: Create list of dirs in a given directory
    // Parameters: $directory - Directory to scan - must be absolute path
    //             $sortorder - Create dirs recursively
    //             $return_files - Retrun a list of files in each directory as well
    // Returns: $arrayList - List of dirs and/or files
    // Author: mygoggie - www.purplescroll.com
    // ------------------------------------------------------------------------------
    function getDirList($directory, $sortorder = 0, $return_files = FALSE)
    {
        // Define some things we need
        $weeds = array('.', '..');

        // Create function scandir() for PHP4 versions - from PHP www site
        if(!function_exists('scandir'))
        {
            function scandir($dir, $sortorder = 0)
            {
                if(is_dir($dir) && $dirlist = @opendir($dir))
                {
                    while(($file = readdir($dirlist)) !== false)
                    {
                        $files[] = $file;
                    }
                    closedir($dirlist);
                    ($sortorder == 0) ? asort($files) : rsort($files); // arsort was replaced with rsort
                    return $files;
                } else return false;
            }
        }

        // Get the sub directories of the give directory
        $scanDirs = scandir($directory, $sortorder);

        // Do some weeding
        $cleaned_directories = array_diff($scanDirs, $weeds);

        foreach($cleaned_directories as $value)
        {
            // Clear php cache
            clearstatcache();

            if($return_files == TRUE)
            {
                if(is_file($directory.DIRECTORY_SEPARATOR.$value))
                {
                    $arrayList[] = $value;
                }

            } else {

                if(is_dir($directory.DIRECTORY_SEPARATOR.$value))
                {
                    $arrayList[] = $value;
                }
            }
        }

    return $arrayList;

    } //EO method getDirList()


    // ------------------------------------------------------------------------------>
    // NAME: getFileCount
    // ------------------------------------------------------------------------------>
    // @function:
    // Get the number of specified files from a directory
    //
    // @parameters:
    // $dirname - The directory to look in
    // $fileExtensions - A list of file extension to look for. Use regex format in list
    //
    // @return:
    // Returns the number of files found
    //
    // @comments:
    // The file extensions are search using regex, so specify the
    // extensions in regex format
    //
    // ------------------------------------------------------------------------------>
    public function getFileCount($dirname=".", $fileExtensions = 'jpe?g|gif|png|bmp')
    {
        $filecnt = 0;

        if ($handle = opendir($dirname)) 
        {
            while (false !== ($file = readdir($handle))) 
            { 
                if (preg_match("/\.($fileExtenstions)$/i",$file)) 
                {
                    $filecnt++;
                }
            }
            closedir($handle); 
        }

        return $filecnt;

    } // EO method getFileCount()


    // ------------------------------------------------------------------------------>
    // NAME: getFileList
    // ------------------------------------------------------------------------------>
    // @function:
    // Get a list of specified files from a directory
    //
    // @parameters:
    // $dirname - The directory to look in
    // $fileExtensions - A list of file extension to look for. Use regex format in list
    // $sort - Must the list be sorted before returning it
    // $sortOrder - The order the list must be sorted by
    //
    // @return:
    // Returns the list of files found
    //
    // @comments:
    // The file extensions are search using regex, so specify the
    // extensions in regex format
    //
    // ------------------------------------------------------------------------------>
    public function getFileList($dirname = ".", $fileExtensions = 'jpe?g|gif|png|bmp', $sort = FALSE, $sortOrder = 'nameASC')
    {

        $arrayList = array(); 

        if ($handle = opendir($dirname)) 
        {
            while (false !== ($file = readdir($handle))) 
            { 
                if (preg_match("/\.({$fileExtensions})$/i",$file)) 
                {
                    $arrayList[] = $file;
                }
            }
            closedir($handle); 
        }

        if($sort == TRUE)
        {
            switch ($sortOrder)
            {
                case 'nameASC':
                    sort($arrayList);
                    reset($arrayList);
                    break;

                case 'nameDESC':
                    rsort($arrayList);
                    reset($arrayList);
                    break;

                case 'oldFIRST':
                    usort($arrayList, array($this, 'sortOld'));
                    reset($arrayList);
                    break;

                case 'newFIRST':
                    usort($arrayList, array($this, 'sortNew'));
                    reset($arrayList);
                    break;

                default:
                    sort($arrayList);
                    reset($arrayList);
            }
        }

        return $arrayList;

    } // EO method getFileList()

} // End of class

// End of PHP file
?>