<?php
/******************************************************************************
* purple_class file                                                           *
*******************************************************************************
*                                                                             *
*   Written by mygoggie                                                       *
*   $Id: purple_class.php                                                     *
*   $Source: www.purplescroll.com/ v$                                         *
*   $Revision: 1.1.0 $                                                        *
*   $Date: 2011-02-10 $                                                       *
*   $Author: mygoggie $                                                       *
*                                                                             *
* @copyright Copyright  2007-2010, purplescroll.com, All rights reserved.    *
*                                                                             *
* This file is part of the purple_cpg_bridge plugin for e107                  *
*                                                                             *
* The purple_cpg_bridge plugin for e107 is free software: you can             *
* redistribute it and/or modify it under the terms of the GNU General         *
* Public License as published by the Free Software Foundation, either         *
* version 3 of the License, or (at your option) any later version.            *
*                                                                             *
* The purple_cpg_bridge plugin for e107 is distributed in the hope that it    *
* will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty *
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with the purple_cpg_bridge plugin for e107.                           *
*                                                                             *
* If not, see <http://www.gnu.org/licenses/>.                                 *
*                                                                             *
******************************************************************************/

// Protect the file from direct access
if(!defined("e107_INIT")){ exit; }

class purple_handlers
{
    //Declare some variables we need in this class
    static private $purple_class_instanceConfig = null;  // The purple_class instance


    // ------------------------------------------------------------------------------
    // PHP5+ Constructor
    // ------------------------------------------------------------------------------
    function __construct()
    {

    // End of constructor
    }


    // ------------------------------------------------------------------------------
    // get_instance method
    // ------------------------------------------------------------------------------
    static public function get_instance($location = NULL)
    {
        // If there is not already an instance of this class, instantiate one.
        if (self::$purple_class_instanceConfig == null){
            self::$purple_class_instanceConfig = new purple_handlers($location);
        }

        return self::$purple_class_instanceConfig;

    } // EO method


    // ------------------------------------------------------------------------------>
    // NAME: trimlrm
    // ------------------------------------------------------------------------------>
    // @Function:
    // Clean string of whitespace - As per PHP www site example
    //
    // @Parameters:
    // $haystack - The string to search
    // $whiteSpaceChars - Chars to search for
    //
    // @Return:
    // Cleaned string
    // ------------------------------------------------------------------------------>
    public function trimlrm($hayStack, $whiteSpaceChars = " \t\n\r\0\x0B")
    {
        $trimmmed = "";
        for ($i = 0; $i < strlen($hayStack); $i++)
        {
            if (strpos($whiteSpaceChars,$hayStack[$i]) === false)
            $trimmed .= $hayStack[$i];
        }

        return $trimmed;

    } //EO method trimlrm()


    // ------------------------------------------------------------------------------>
    // NAME: fixWinslowsPath
    // ------------------------------------------------------------------------------>
    // @Function:
    // Fix Windows path
    //
    // @Parameters:
    // $path - The path to fix stupid winslows slashes
    //
    // @Return:
    // Cleaned path
    // ------------------------------------------------------------------------------>
    public function fixWinslowsPath($path) 
    {
        $path = $this->backslashToForwardslash(preg_replace('/(\\\\)/','\\', $path));
        return $path;
    } // EO method fixWinslowsPath()


    // ------------------------------------------------------------------------------>
    // NAME: backslashToForwardslash
    // ------------------------------------------------------------------------------>
    // @Function:
    // Replace a backslash with a forward slash
    //
    // @Parameters:
    // $value - The string to convert
    //
    // @Return:
    // Cleaned string
    // ------------------------------------------------------------------------------>
    public function backslashToForwardslash($value)
    {
        return str_replace('\\',DIRECTORY_SEPARATOR, $value);
    } // EO method backslashToForwardslash()


    // ------------------------------------------------------------------------------>
    // NAME: forwardslashToBackslash
    // ------------------------------------------------------------------------------>
    // @Function:
    //Replace a forward slash with a backslash
    //
    // @Parameters:
    // $value - The string to convert
    //
    // @Return:
    // Cleaned string
    // ------------------------------------------------------------------------------>
    public function forwardslashToBackslash($value)
    {
        return str_replace("/", DIRECTORY_SEPARATOR, $value);
    } // EO method forwardslashToBackslash()


    // ------------------------------------------------------------------------------>
    // NAME: getSiteID
    // ------------------------------------------------------------------------------>
    // @Function:
    // Append a trailing forward slash
    //
    // @Parameters:
    // $value - The string to append
    //
    // @Return:
    // Cleaned string
    // ------------------------------------------------------------------------------>
    public function addTrailingSlash($value) 
    {
        if(preg_match('@^.*[^/]{1}$@i', $value))
        {
            $value .= DIRECTORY_SEPARATOR;
        }

        return $value;
    } // EO method addTrailingSlash()


    // ------------------------------------------------------------------------------>
    // NAME: removePrependedSlash
    // ------------------------------------------------------------------------------>
    // @Function:
    // Remove prepended forward slash
    //
    // @Parameters:
    // $value - The string to strip
    //
    // @Return:
    // Cleaned string
    // ------------------------------------------------------------------------------>
    public function removePrependedSlash($value)
    {
        $value = $this->backslashToForwardslash($value);

        if(preg_match('@^.+/$@i', $value))
        {
            $value = ltrim($value, '/');
        }

        return $value;
    } // EO method removePrependedSlash()

    // ------------------------------------------------------------------------------>
    // NAME: removeTrailingSlash
    // ------------------------------------------------------------------------------>
    // @Function:
    // Remove trailing forward slash
    //
    // @Parameters:
    // $value - The string to strip
    //
    // @Return:
    // Cleaned string
    // ------------------------------------------------------------------------------>
    public function removeTrailingSlash($value)
    {
        $value = $this->backslashToForwardslash($value);

        if(preg_match('@^.+/$@i', $value))
        {
            $value = substr($value, 0, strlen($value)-1);
        }

        return $value;
    } // EO method removeTrailingSlash()


    // ------------------------------------------------------------------------------>
    // NAME: removeValueFromArray
    // ------------------------------------------------------------------------------>
    // @Function:
    // Remove a value that contains a specified string from an array
    // If the string is empty (the default) it removes empty values from an array
    //
    // @Parameters:
    // $array - The array to search
    // $string - Chars to search for
    //
    // @Return:
    // Cleaned array
    // ------------------------------------------------------------------------------>
    public function removeValueFromArray($array, $string="")
    {
        foreach ($array as $index => $value)
        {
            if ($value == $string)
            {
                unset($array[$index]);
            }

            if (!empty($string) && stristr($value, $string))
            {
                unset($array[$index]);
            }
        }

        $array = array_values($array);

        return $array;

    } // EO method removeValueFromArray()


    // ------------------------------------------------------------------------------>
    // NAME: array_diff_no_cast
    // ------------------------------------------------------------------------------>
    // @Function:
    // Get the difference between two multi dimensional arrays
    // As per www.php.net
    //
    // @Parameters:
    // $ar1 - The array to reference against
    // $ar2 - The smaller array
    //
    // @Return:
    // Array containing all elements not in $ar2
    // ------------------------------------------------------------------------------>
    public function array_diff_no_cast(&$ar1, &$ar2)
    {
        $diff = Array();

        foreach ($ar1 as $key => $val1)
        {
            if (array_search($val1, $ar2) === FALSE)
            {
                $diff[$key] = $val1;
            }
        }

        return $diff;
    } // EO method array_diff_no_cast()


// End of class
}

// End of php file
?>