<?php
/******************************************************************************
* purple_userclass_class file                                                 *
*******************************************************************************
*                                                                             *
*   Written by mygoggie                                                       *
*   $Id: purple_userclass_class.php                                           *
*   $Source: www.purplescroll.com/ v$                                         *
*   $Revision: 1.1.0 $                                                        *
*   $Date: 2011-02-10 $                                                       *
*   $Author: mygoggie $                                                       *
*                                                                             *
* @copyright Copyright  2007-2010, purplescroll.com, All rights reserved.    *
*                                                                             *
* This file is part of the purple_login plugin for e107                       *
*                                                                             *
* The purple_login plugin for e107 is free software: you can                  *
* redistribute it and/or modify it under the terms of the GNU General         *
* Public License as published by the Free Software Foundation, either         *
* version 3 of the License, or (at your option) any later version.            *
*                                                                             *
* The purple_login plugin for e107 is distributed in the hope that it         *
* will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty *
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with the purple_login plugin for e107.                                *
*                                                                             *
* If not, see <http://www.gnu.org/licenses/>.                                 *
*                                                                             *
******************************************************************************/
// Protect the file from direct access
if(!defined("e107_INIT")){ exit; }

class purple_userclass
{

    static private $purple_userclass_instanceConfig = null;

    // ------------------------------------------------------------------------------
    // PHP5+ Constructor
    // ------------------------------------------------------------------------------
    function __construct()
    {

    // End of constructor
    }

    // ------------------------------------------------------------------------------
    // get_instance method
    // ------------------------------------------------------------------------------
    static public function get_instance($location = NULL)
    {
        // If there is not already an instance of this class, instantiate one.
        if (self::$purple_userclass_instanceConfig == NULL){
            self::$purple_userclass_instanceConfig = new purple_userclass($location);
        }
 
        return self::$purple_userclass_instanceConfig;

    // End of method
    }


    // ------------------------------------------------------------------------------>
    // NAME: getUserclassList
    // ------------------------------------------------------------------------------>
    // Function:
    // Get the userclass values from the DB and list only the userclasses specified
    // This method is an extension on the method in the userclass_class.php file
    //
    // Parameters:
    // none
    //
    // Return:
    // Returns a list of the userclasses
    // ------------------------------------------------------------------------------>
    function getUserclassList()
    {

        //Declare variables
        global $sql;

        // Get the userclass values from the DB
        if ($sql->db_Select("userclass_classes", "userclass_id, userclass_name", "ORDER BY userclass_name", "nowhere"))
		{
			while($row = $sql->db_Fetch())
			{
				$userclassList[$row['userclass_id']] = $row['userclass_name'];
			}

            return $userclassList;
        }

        // No userclass found
        return FALSE;

    } // EO method getUserclassList()


    // ------------------------------------------------------------------------------>
    // NAME: buildUserclassList
    // ------------------------------------------------------------------------------>
    // Function:
    // Get the userclass values from the DB and list only the userclasses specified
    // This method is an extension on the method in the userclass_class.php file
    //
    // Parameters:
    // $fieldname - name of the field e.g. name =""
    // $class - class to use for the field
    // $id - id to use for the field
    // $selectedValue - selected value
    // $defaults - remove the default userclassses as per the file userclass_class.php
    //
    // Return:
    // Returns a selection list string
    // ------------------------------------------------------------------------------>
    function buildUserclassList($id='', $fieldname='', $class='', $selectedValue = 0, $removeList='defaults', $message='')
    {
        // Get list of userclasses
        $userclassList = $this->getUserclassList();

        switch ($removeList)
        {
            case 'defaults':
                $arrayToRemove = explode(',', 'PRIVATEFORUM1,PRIVATEMENU');
                break;

            case 'none':
                $arrayToRemove = array();
                break;

            default:
                $arrayToRemove = explode(',',$removeList);
        }

        // Check if userclass values must be removed
        if (!empty($arrayToRemove))
        {
            // Remove the specified userclasses from the array $userclassList
            $userclassList = array_diff($userclassList, $arrayToRemove);
        }

        $text = "<select id='{$id}' name='{$fieldname}' class='{$class}'>";

        if (!empty($message))
        {
            $text .= "<option value='none'>{$message}</option>";
        }
        
        foreach( $userclassList as $key=>$value )
        {
            $s = ($selectedValue == $key) ?  "selected='selected'" : "";
            $text .= "<option value='{$key}' {$s} >{$value}</option>";
        }
        
        $text .= "</select>\n";

        return $text;

    } // End of method buildUserclassList()


    // ------------------------------------------------------------------------------>
    // NAME: new_userclass
    // ------------------------------------------------------------------------------>
    // Function:
    // Create a new userclass
    //
    // Parameters:
    // $userclass - Name of the userclass to be created
    // $userclass_description - Description of the userclass
    // $userclass_manager - Who can manage the userclass
    //
    // Return:
    // Returns TRUE on success
    // ------------------------------------------------------------------------------>
    function new_userclass($userclass, $userclass_description, $userclass_manager)
    {
		// Clear the variables
		unset($flag);

        // Declare variables
        global $sql;

		if (FALSE !== $userclass && (getperms('0') || check_class($userclass)))
		{
			$i = 1;

			while ($sql->db_Select('userclass_classes', '*', "userclass_id='".$i."' ") && $i < 255)
			{
				$i++;
			}

			if ($i < 245)
			{
				$sql->db_Insert("userclass_classes", $i.", '{$userclass}', '{$userclass_description}',{$userclass_manager}");

				// Set flag
				$flag = TRUE;
			}
        }

        return $flag;

    } // End of method new_userclass()


    // ------------------------------------------------------------------------------>
    // NAME: add_userclass
    // ------------------------------------------------------------------------------>
    // Function:
    // Adds a userclass to an array of users
    //
    // Parameters:
    // $userclass_ID - ID of the userclass to be added
    // $arrayUsers - The key is the user's ID and the value is the comma separated list of the classes to which the user currently belongs
    //
    // Return:
    // Returns TRUE on success
    // ------------------------------------------------------------------------------>
	function add_userclass($userclass_ID, $arrayUsers)
	{
		// Declare variables
		global $tp;
		global $sql;

		foreach($arrayUsers as $uid => $currentClass)
		{
			if ($currentClass)
			{
				$newarray = array_unique(array_merge(explode(',', $currentClass), array($userclass_ID)));
				$new_userclass = implode(',', $newarray);
			}
			else
			{
				$new_userclass = $userclass_ID;
            }

            $sql->db_Update('user', "user_class='".$tp -> toDB($new_userclass, true)."' WHERE user_id=".intval($uid));
		}
	} // End of method add_userclas()


    // ------------------------------------------------------------------------------>
    // NAME: delete_userclass
    // ------------------------------------------------------------------------------>
    // Function:
    // Delete a userclass from the table userclass_classes as well as from any users
    //
    // Parameters:
    // $userclass - ID of the userclass to be deleted
    //
    // Return:
    // Returns nothing
    // ------------------------------------------------------------------------------>
    function delete_userclass($userclass_ID)
    {
        // Declare variables
        global $sql;
        global $pref;

//print "<br />In delete userclass. Userclass = ".$userclass_ID;

        // Delete the userclass from the table
        $sql->db_Delete('userclass_classes', "userclass_id=".$userclass_ID);

        // Remove the userclass from all the users
        $this->remove_userclass($userclass_ID);

        if (isset($pref['frontpage'][$userclass_ID]))
        {
            unset($pref['frontpage'][$userclass_ID]);
            save_prefs();
        }

    } // EO of method delete_userclass()


    // ------------------------------------------------------------------------------>
    // NAME: remove_userclass
    // ------------------------------------------------------------------------------>
    // Function:
    // Remove a userclass from an array of users
    //
    // Parameters:
    // $userclass - ID of the userclass to be removed from the users
    //
    // Return:
    // Returns nothing
    // ------------------------------------------------------------------------------>
    function remove_userclass($userclass_ID)
    {
        // Declare variables
        global $tp;
        global $sql;
        $arrayUserInfo = array();

//print "<br /> Removing userclass from users : ".$userclass_ID;

        if ($sql->db_Select('user', 'user_id, user_class', "user_class = '{$userclass_ID}' OR user_class REGEXP('^{$userclass_ID},') OR user_class REGEXP(',{$userclass_ID},') OR user_class REGEXP(',{$userclass_ID}$')"))
        {
            // Built the list of users
            while ($row = $sql->db_Fetch())
            {
                $arrayUserInfo[$row['user_id']] = $row['user_class'];
            }

//print "<br />In delete userclass. Array of users = ";var_dump($arrayUserInfo);
            // Remove the userclass
            foreach($arrayUserInfo as $user_ID => $curclass)
            {
                $newarray = array_diff(explode(',', $curclass), array('', $userclass_ID));
//print "<br />New array of userclass= ";var_dump($newarray);
                $new_userclass = implode(',', $newarray);
//print "<br />New userclass= ";var_dump($new_userclass);
//print "<br />Userclass to DB = ".$tp->toDB($new_userclass, true);
//print "<br />User ID= ".intval($user_ID);
                $sql->db_Update('user', "user_class='".$tp->toDB($new_userclass, true)."' WHERE user_id=".intval($user_ID));

                // Build new userclass per user
                $newArrayUserInfo[$user_ID] = $new_userclass;

            }

            
        }
//print "<br />New user array= ";var_dump($newArrayUserInfo);
        
        return $newArrayUserInfo;
    } // EO of method class_remove()


// End of class
}

// End of php file
?>